@@ -5,8 +5,20 @@ from .page_creator import PageCreator
 from .point import Point
 from .searchable_pdf import save_searchable_pdf
 from .segment import Segment
-from .text_block import Character, Document, Line, Page, Paragraph, TextBlock, Word
+from .text_block import (
+    Character,
+    Document,
+    Page,
+    Area,
+    Line,
+    Paragraph,
+    TextBlock,
+    Word,
+    Table,
+    TableColumn,
+    TableCell,
+)
 from .text_block_splitter import TextBlockSplitter
 from .textract_parser import TextractParser
 
-__version__ = "1.0.200"
+__version__ = "1.0.214"
