@@ -6,8 +6,9 @@ from .segment import Segment
 class BoundingBox:
     """
     A bounding box is a rectangle that has a left, top, right, and bottom.
-    The coordinates of a box are meant to be normalized pdf coordinates i.e. the origin is at the bottom left of the page,
-    the x-axis is horizontal and increases to the right, and the y-axis is vertical and increases upwards and the page coordinates
+    The coordinates of a box are meant to be normalized pdf coordinates i.e.
+    the origin is at the bottom left of the page, the x-axis is horizontal and increases
+    to the right, and the y-axis is vertical and increases upwards and the page coordinates
     is left = 0, top = 1, right = 1, bottom = 0.
     """
 
@@ -18,6 +19,7 @@ class BoundingBox:
         self.bottom = bottom
         self.height = self.top - self.bottom
         self.width = self.right - self.left
+        self.validate_args()
 
     def validate_args(self):
         """Validate the arguments of the bounding box."""
@@ -102,11 +104,17 @@ class BoundingBox:
         bottom = center.y - ver_shift
         return BoundingBox(left, top, right, bottom)
 
+    def horizontal_intersect(self, other: "BoundingBox") -> bool:
+        """Check if the bounding boxes intersect horizontally."""
+        return self.left <= other.right and other.left <= self.right
+
+    def vertical_intersect(self, other: "BoundingBox") -> bool:
+        """Check if the bounding boxes intersect vertically."""
+        return self.bottom <= other.top and other.bottom <= self.top
+
     def intersect(self, other: "BoundingBox") -> bool:
         """Check if the bounding boxes intersect."""
-        hor_intersection = self.left <= other.right and other.left <= self.right
-        ver_intersection = self.bottom <= other.top and other.bottom <= self.top
-        return hor_intersection and ver_intersection
+        return self.horizontal_intersect(other) and self.vertical_intersect(other)
 
     def contains_point(self, point: Point) -> bool:
         """Check if the bounding box contains the point."""
@@ -136,7 +144,10 @@ class BoundingBox:
         )
 
     def __str__(self):
-        return f"BoundingBox(l={self.left:.3g}, t={self.top:.3g}, r={self.right:.3g}, b={self.bottom:.3g})"
+        return (
+            f"BoundingBox(l={self.left:.3g}, t={self.top:.3g}, "
+            f"r={self.right:.3g}, b={self.bottom:.3g})"
+        )
 
     def __repr__(self):
         return str(self)
@@ -146,7 +157,8 @@ class BoundingBox:
         bounding_boxes: list["BoundingBox"],
     ) -> Optional["BoundingBox"]:
         """Compose a list of bounding boxes into a single bounding box.
-        The resulting bounding box is the smallest bounding box that contains all the bounding boxes in the list.
+        The resulting bounding box is the smallest bounding box that contains all
+        the bounding boxes in the list.
         """
         if not bounding_boxes:
             return None
@@ -158,7 +170,9 @@ class BoundingBox:
 
     @staticmethod
     def split_bounding_box(
-        bounding_box: "BoundingBox", num_boxes: int
+        bounding_box: "BoundingBox",
+        num_boxes: int,
+        weights: list[float] = None,
     ) -> list["BoundingBox"]:
         """Split a bounding box into a list of bounding boxes.
         The bounding boxes are split evenly along the width of the bounding box."""
@@ -166,21 +180,30 @@ class BoundingBox:
             raise ValueError("num_boxes must be greater than 0")
         if num_boxes == 1:
             return [bounding_box]
+        if weights is None:
+            weights = [1 / len(num_boxes) for _ in range(num_boxes)]
+        else:
+            total_weight = sum(weights)
+            weights = weights / total_weight
+
         width = bounding_box.get_width()
         left = bounding_box.left
         top = bounding_box.top
-        right = bounding_box.right
         bottom = bounding_box.bottom
+        char_widths = weights * width
         bounding_boxes = []
+        char_left = left
         for i in range(num_boxes):
+            char_right = char_left + char_widths[i]
             bounding_boxes.append(
                 BoundingBox(
-                    left + i * width / num_boxes,
+                    char_left,
                     top,
-                    left + (i + 1) * width / num_boxes,
+                    char_right,
                     bottom,
                 )
             )
+            char_left = char_right
         return bounding_boxes
 
     def draw_on_image(
