@@ -1,7 +1,5 @@
 import re
-
 import attr
-
 from ..bounding_box import BoundingBox
 from ..constants import (
     PARAGRAPH_HORIZONTAL_SCALE,
@@ -146,7 +144,7 @@ class ParagraphExtractor:
     ) -> list[ParagraphIndexed]:
         paragraphs = []
         for cc in ccs:
-            lines_indexes = [node.data for node in cc]
+            lines_indexes = sorted([node.data for node in cc])
             bounding_boxes = [self.lines[i].bounding_box for i in lines_indexes]
             bounding_box = BoundingBox.compose_bounding_boxes(bounding_boxes)
             paragraphs.append(ParagraphIndexed(lines_indexes, bounding_box))
@@ -163,6 +161,13 @@ class ParagraphExtractor:
             paragraphs.append(paragraph)
         return paragraphs
 
+    def extract_paragraphs_temp(self) -> list[Paragraph]:
+        if not self.lines:
+            return []
+        single_paragraph = Paragraph(children=self.lines)
+        single_paragraph.set_bounding_box()
+        return [single_paragraph]
+
     def extract_paragraphs(self) -> list[Paragraph]:
         if not self.lines:
             return []
