@@ -1,7 +1,5 @@
-AXIS_DELTA_NET_PARTITION = 100
 ANGLE_DEGREE_THRESHOLD = 3
-
-WIDTH_SCALE = 0.05
-HEIGHT_SCALE = 0.05
 NUM_SAMPLED_POINTS_ON_LINE = 10
 NUM_SAMPLED_POINTS_ON_ORTHOGONAL_LINE = 10
+SPATIAL_GRID_SIZE = 20
+GRID_COVER_RATIO_THRESHOLD = 0.5
